require 'test/unit'
require 'rjb'

puts "start RJB(#{Rjb::VERSION}) test"
class TestRjb < Test::Unit::TestCase
  include Rjb
  def setup
    load
  end

  def tearDown
    unload
  end

  def test_metaclass
    cls = import('java.lang.Class')
    assert_equal('java.lang.Class', cls._classname)
    assert_equal('java.lang.Class', cls.getName)
    assert_equal(17, cls.getModifiers)
  end

  def test_scalar
    cls = import('java.lang.String')
    assert_equal('java.lang.Class', cls._classname)
    assert_equal('class java.lang.String', cls.toString)
    str = cls.new
    assert_equal('java.lang.String', str._classname)
    assert_equal(0, str.length)
    assert_equal('', str.toString)
    str = cls.new_with_sig('Ljava.lang.String;', "abcde")
    # result integer
    assert_equal(5, str.length)
    # result string
    assert_equal('abcde', str.toString)
    # argument test
    # char
    assert_equal('abxde', str.replace(?c, ?x))
    # string
    assert_equal('abdxe', str.replaceAll('cd', 'dx'))
    # int
    assert_equal('bc', str.substring(1, 3))
    assert_equal('e', str.substring(4))
    # float with static
    assert_equal('5.23', cls._invoke('valueOf', 'F', 5.23))
    assert_equal('25.233', cls._invoke('valueOf', 'D', 25.233))
    # rjb object (String)
    str2 = cls.new_with_sig('Ljava.lang.String;', 'fghijk')
    assert_equal('abcdefghijk', str.concat(str2))
    # rjb object other (implicit toString call is Rjb feature)
    jint = import('java.lang.Integer')
    i = jint.new_with_sig('I', 35901)
    assert_equal('abcde35901', str.concat(i))
    # result boolean and argument is rjb object
    assert_equal(false, i.equals(str))
    assert_equal(false, str.equals(i))
    assert_equal(true, str.equals("abcde"))
    assert_equal(true, str.equals(str))
    # long
    lng = import('java.lang.Long')
    l = lng.new_with_sig('J', -9223372036854775808)
    assert_equal(-9223372036854775808, l.longValue)
    l = lng.new_with_sig('J', 9223372036854775807)
    assert_equal(9223372036854775807, l.longValue)
    # double
    dbl = import('java.lang.Double')
    d = dbl.new_with_sig('D', 1234.5678901234567890)
    assert_equal(1234.5678901234567890, d.doubleValue)
    # byte
    byt = import('java.lang.Byte')
    b = byt.new_with_sig('B', 13)
    assert_equal(13, b.byteValue)
    # float
    flt = import('java.lang.Float')
    f = flt.new_with_sig('F', 13.5)
    assert_equal(13.5, f.floatValue)
    # short
    sht = import('java.lang.Short')
    s = sht.new_with_sig('S', 1532)
    assert_equal(1532, s.shortValue)
    chr = import('java.lang.Character')
    c = chr.new_with_sig('C', ?A)
    assert_equal(?A, c.charValue)
  end

  def test_array
    cls = import('java.lang.String')
    str = cls.new_with_sig('[C', [?a, ?b, ?c, ?d, ?e, ?c, ?f, ?c, ?g])
    assert_equal('abcdecfcg', str.toString)
    # conv string array
    splt = str.split('c')
    assert(Array === splt)
    assert_equal(4, splt.size)
    assert_equal('ab', splt[0])
    assert_equal('g', splt[3])
    # conv byte array to (ruby)string
    ba = str.getBytes
    assert_equal('abcdecfcg', ba)
    # conv char array to array(int)
    ca = str.toCharArray
    assert_equal([?a, ?b, ?c, ?d, ?e, ?c, ?f, ?c, ?g], ca)
  end

  def test_importobj()
    sys = import('java.lang.System')
    props = sys.getProperties
    assert_equal('java.util.Properties', props._classname)
    assert_equal(Dir::pwd, props.getProperty('user.dir').gsub('\\', '/'))
    boolean = import('java.lang.Boolean')
    assert_equal(boolean.valueOf(true).booleanValue(), true)
    assert_equal(boolean.valueOf(false).booleanValue(), false)
    assert_equal(boolean.valueOf('true').booleanValue(), true)
    assert_equal(boolean.valueOf('false').booleanValue(), false)
  end

  def test_importobjarray()
    jarray = import('java.util.ArrayList')
    a = jarray.new()
    jint = import('java.lang.Integer')
    a.add(jint.new_with_sig('I', 1))
    a.add(jint.new_with_sig('I', 2))
    a.add(jint.new_with_sig('I', 3))
    oa = a.toArray
    assert_equal(3, oa.size)
    assert_equal(1, oa[0].intValue)
    assert_equal(2, oa[1].intValue)    
    assert_equal(3, oa[2].intValue)    
  end

  def test_constants()
    lng = import('java.lang.Long')
    assert_equal(0x7fffffffffffffff, lng.MAX_VALUE)
    assert_equal(-9223372036854775808, lng.MIN_VALUE)
  end

  class TestIter
    def initialize()
      @i = 5
    end
    def hasNext()
      @i > 0
    end
    def next()
      @i -= 1
      @i.to_s
    end
  end

  def test_newobject()
    it = TestIter.new
    it = bind(it, 'java.util.Iterator')
    test = import('jp.co.infoseek.hp.arton.rjb.Test')
    a = test.new
    assert("43210", a.concat(it))
  end

  class TestComparator
    def compare(o1, o2)
      o1.to_i - o2.to_i
    end
    def equals(o)
      o == self
    end
  end

  def test_comparator
    cp = TestComparator.new
    cp = bind(cp, 'java.util.Comparator')
    test = import('jp.co.infoseek.hp.arton.rjb.Test')
    a = test.new
    assert(0, a.check(cp, 123, 123))
    assert(5, a.check(cp, 81, 76))
    assert(-5, a.check(cp, 76, 81))
  end

  # assert_raise is useless in this test, because NumberFormatException may be defined in
  # its block.
  def test_exception()
    it = import('java.lang.Integer')
    begin
      it.parseInt('blabla')
      flunk('no exception')
    rescue NumberFormatException => e
      # OK
    end
  end

  def test_field()
    point = import('java.awt.Point')
    pnt = point.new(11, 13)
    assert_equal(11, pnt.x)
    assert_equal(13, pnt.y)
    pnt.x = 32
    assert_equal(32, pnt.x)
  end

end

